@@ -6,11 +6,12 @@ from __future__ import absolute_import
 import itertools
 import os.path
 
-from pex.dist_metadata import is_wheel
+from pex.dist_metadata import Distribution, is_wheel
 from pex.orderedset import OrderedSet
 from pex.rank import Rank
 from pex.third_party.packaging.tags import Tag, parse_tag
 from pex.typing import TYPE_CHECKING, cast, overload
+from pex.wheel import WHEEL
 
 if TYPE_CHECKING:
     from typing import (
@@ -66,7 +67,13 @@ class CompatibilityTags(object):
 
     @classmethod
     def from_wheel(cls, wheel):
-        # type: (Text) -> CompatibilityTags
+        # type: (Union[Text, Distribution]) -> CompatibilityTags
+
+        if isinstance(wheel, Distribution):
+            if not is_wheel(wheel.location):
+                return cls(tags=WHEEL.from_distribution(wheel).tags)
+            wheel = wheel.location
+
         if not is_wheel(wheel):
             raise ValueError(
                 "Can only calculate wheel tags from a filename that ends in .whl per "
@@ -74,6 +81,7 @@ class CompatibilityTags(object):
                     wheel=wheel
                 )
             )
+
         wheel_stem, _ = os.path.splitext(os.path.basename(wheel))
         # Wheel filename format: https://peps.python.org/pep-0427/#file-name-convention
         # `{distribution}-{version}(-{build tag})?-{python tag}-{abi tag}-{platform tag}.whl`
@@ -86,6 +94,7 @@ class CompatibilityTags(object):
                     pattern=pattern, wheel=wheel
                 )
             )
+
         return cls(tags=tuple(parse_tag("-".join(wheel_components[-3:]))))
 
     @classmethod
