@@ -7,13 +7,14 @@ import os
 import sys
 from collections import OrderedDict, defaultdict, deque
 
-from pex import toml
+from pex import pex_warnings, toml
 from pex.artifact_url import RANKED_ALGORITHMS, VCS, ArtifactURL, Fingerprint, VCSScheme
 from pex.common import pluralize
 from pex.compatibility import text, urlparse
 from pex.dependency_configuration import DependencyConfiguration
 from pex.dist_metadata import Constraint, Requirement
 from pex.exceptions import production_assert
+from pex.interpreter_implementation import InterpreterImplementation
 from pex.network_configuration import NetworkConfiguration
 from pex.orderedset import OrderedSet
 from pex.pep_425 import CompatibilityTags, RankedTag
@@ -27,16 +28,17 @@ from pex.resolve.locked_resolve import (
     LockedRequirement,
     LockedResolve,
     Resolved,
-    TargetSystem,
     UnFingerprintedLocalProjectArtifact,
     UnFingerprintedVCSArtifact,
     VCSArtifact,
 )
+from pex.resolve.lockfile import tarjan
 from pex.resolve.lockfile.requires_dist import remove_unused_requires_dist
 from pex.resolve.lockfile.subset import Subset, SubsetResult
 from pex.resolve.requirement_configuration import RequirementConfiguration
 from pex.resolve.resolved_requirement import Pin
 from pex.resolve.resolver_configuration import BuildConfiguration
+from pex.resolve.target_system import TargetSystem, UniversalTarget
 from pex.result import Error, ResultError, try_
 from pex.sorted_tuple import SortedTuple
 from pex.targets import Target, Targets
@@ -77,9 +79,15 @@ else:
 def _calculate_marker(
     project_name,  # type: ProjectName
     dependants_by_project_name,  # type: Mapping[ProjectName, OrderedSet[Tuple[ProjectName, Optional[Marker]]]]
+    seen=None,  # type: Optional[Set[ProjectName]]
 ):
     # type: (...) -> Optional[Marker]
 
+    visited = set() if seen is None else seen
+    if project_name in visited:
+        return None
+    visited.add(project_name)
+
     dependants = dependants_by_project_name.get(project_name)
     if not dependants:
         return None
@@ -93,7 +101,9 @@ def _calculate_marker(
         and_markers = OrderedSet()  # type: OrderedSet[str]
         if marker:
             and_markers.add(str(marker))
-        guard_marker = _calculate_marker(dependant_project_name, dependants_by_project_name)
+        guard_marker = _calculate_marker(
+            dependant_project_name, dependants_by_project_name, seen=visited
+        )
         if guard_marker:
             and_markers.add(str(guard_marker))
 
@@ -166,53 +176,105 @@ def _elide_extras(marker):
     return marker
 
 
-def _to_environment(system):
-    # type: (TargetSystem.Value) -> str
+def _implementation_marker(
+    implementation,  # type: InterpreterImplementation.Value
+    extra_marker=None,  # type: Optional[Marker]
+):
+    # type: (...) -> str
+    platform_python_implementation = "platform_python_implementation == '{implementation}'".format(
+        implementation=implementation
+    )
+    if not extra_marker:
+        return platform_python_implementation
+    return "{platform_python_implementation} and {extra_marker}".format(
+        platform_python_implementation=platform_python_implementation, extra_marker=extra_marker
+    )
+
+
+def _to_environment(
+    system,  # type: TargetSystem.Value
+    implementation=None,  # type: Optional[InterpreterImplementation.Value]
+    extra_marker=None,  # type: Optional[Marker]
+):
+    # type: (...) -> str
+
     if system is TargetSystem.LINUX:
-        return "platform_system == 'Linux'"
+        platform_system = "platform_system == 'Linux'"
     elif system is TargetSystem.MAC:
-        return "platform_system == 'Darwin'"
+        platform_system = "platform_system == 'Darwin'"
     else:
         production_assert(system is TargetSystem.WINDOWS)
-        return "platform_system == 'Windows'"
+        platform_system = "platform_system == 'Windows'"
+
+    if not implementation:
+        if not extra_marker:
+            return platform_system
+        return "{platform_system} and {extra_marker}".format(
+            platform_system=platform_system, extra_marker=extra_marker
+        )
+
+    return "{platform_system} and {platform_python_implementation}".format(
+        platform_system=platform_system,
+        platform_python_implementation=_implementation_marker(
+            implementation, extra_marker=extra_marker
+        ),
+    )
 
 
 def convert(
     root_requirements,  # type: Iterable[Requirement]
     locked_resolve,  # type: LockedResolve
     output,  # type: IO[bytes]
-    requires_python=None,  # type: Optional[str]
-    target_systems=(),  # type: Iterable[TargetSystem.Value]
+    universal_target=None,  # type: Optional[UniversalTarget]
     subset=(),  # type: Iterable[DownloadableArtifact]
     include_dependency_info=True,  # type bool
 ):
     # type: (...) -> None
 
-    locked_resolve = remove_unused_requires_dist(
-        resolve_requirements=root_requirements,
-        locked_resolve=locked_resolve,
-        requires_python=[requires_python] if requires_python else [],
-        target_systems=target_systems,
-    )
-
     pylock = OrderedDict()  # type: OrderedDict[str, Any]
     pylock["lock-version"] = "1.0"  # https://peps.python.org/pep-0751/#lock-version
 
-    if target_systems:
+    if universal_target:
         # https://peps.python.org/pep-0751/#environments
         #
-        # TODO: We just stick to mapping `--target-system` into markers currently but this should
-        #  probably include the full marker needed to rule out invalid installs, like Python 2.7
-        #  attempting to install a lock with only Python 3 wheels.
-        pylock["environments"] = sorted(_to_environment(system) for system in target_systems)
-    if requires_python:
-        # https://peps.python.org/pep-0751/#requires-python
-        #
-        # TODO: This is currently just the `--interpreter-constraint` for `--style universal` locks
-        #  but it should probably be further refined (or purely calculated for non universal locks)
-        #  from locked project requires-python values and even more narrowly by locked projects with
-        #  only wheel artifacts by the wheel tags.
-        pylock["requires-python"] = requires_python
+        # TODO: We just stick to mapping `--target-system` and any `--interpreter-constraint` that
+        #  have an implementation specified into markers currently but this should probably include
+        #  the full marker needed to rule out invalid installs, like Python 2.7 attempting to
+        #  install a lock with only Python 3 wheels.
+        if universal_target.systems:
+            pylock["environments"] = sorted(
+                _to_environment(
+                    system,
+                    implementation=universal_target.implementation,
+                    extra_marker=locked_resolve.marker,
+                )
+                for system in universal_target.systems
+            )
+        elif universal_target.implementation:
+            pylock["environments"] = [
+                _implementation_marker(
+                    universal_target.implementation, extra_marker=locked_resolve.marker
+                )
+            ]
+        elif locked_resolve.marker:
+            pylock["environments"] = [str(locked_resolve.marker)]
+
+        if universal_target.requires_python:
+            if len(universal_target.requires_python) > 1:
+                # TODO(John Sirois): Provide a better error message. We could guide on OR
+                #  to and AND paired with != to remove disjoint portions of the range.
+                raise ValueError(
+                    "Can only convert a lock file with a single interpreter constraint."
+                )
+            requires_python = universal_target.requires_python[0]
+
+            # https://peps.python.org/pep-0751/#requires-python
+            #
+            # TODO: This is currently just any `--interpreter-constraint` specifiers for
+            #  `--style universal` locks but it should probably be further refined (or purely
+            #  calculated for non universal locks) from locked project requires-python values and
+            #  even more narrowly by locked projects with only wheel artifacts by the wheel tags.
+            pylock["requires-python"] = str(requires_python)
 
     # TODO: These 3 assume a `pyproject.toml` is the input source for the lock. It almost never is
     #  for current Pex lock use cases. Figure out if there is anything better that can be done.
@@ -242,6 +304,11 @@ def convert(
         if req.url and isinstance(parse_requirement_string(str(req)), URLRequirement)
     }  # type: Dict[ProjectName, Requirement]
 
+    locked_resolve = remove_unused_requires_dist(
+        resolve_requirements=root_requirements,
+        locked_resolve=locked_resolve,
+        universal_target=universal_target,
+    )
     dependants_by_project_name = defaultdict(
         OrderedSet
     )  # type: DefaultDict[ProjectName, OrderedSet[Tuple[ProjectName, Optional[Marker]]]]
@@ -499,11 +566,12 @@ class ParseContext(object):
         self,
         key,  # type: str
         default=None,  # type: Optional[str]
+        footer_message=None,  # type: Optional[str]
     ):
         # type: (...) -> str
         # The cast is of Python 2. The return type will actually be `unicode` in that case, but it
         # doesn't matter since there will be no further runtime type checks above this call.
-        return cast(str, self.get(key, text, default=default))
+        return cast(str, self.get(key, text, default=default, footer_message=footer_message))
 
     def get_array_of_strings(
         self,
@@ -584,14 +652,16 @@ class ParseContext(object):
         key,  # type: str
         item_type,  # type: Type[_T]
         default=None,  # type: Optional[_T]
+        footer_message=None,  # type: Optional[str]
     ):
         # type: (...) -> _T
         value = self.table.get(key, None)
         if value is None:
             if default is None:
-                raise ResultError(
-                    self.error("A value for {key} is required.".format(key=self.subpath(key)))
-                )
+                msg_lines = ["A value for {key} is required.".format(key=self.subpath(key))]
+                if footer_message:
+                    msg_lines.append(footer_message)
+                raise ResultError(self.error(os.linesep.join(msg_lines)))
             return default
         if not isinstance(value, item_type):
             raise ResultError(
@@ -788,43 +858,11 @@ class PackageIndex(object):
         return self._indexed_packages_by_name.get(project_name)
 
 
-def spec_matches(
-    spec,  # type: Any
-    package_data,  # type: Any
-):
-    # type: (...) -> bool
-
-    if isinstance(spec, dict) and isinstance(package_data, dict):
-        for key, value in spec.items():
-            if not spec_matches(value, package_data.get(key)):
-                return False
-        return True
-
-    if isinstance(spec, list) and isinstance(package_data, list):
-        # All instances of lists in packages are currently array of tables:
-        # + dependencies
-        # + wheels
-        # + attestation-identities
-        #
-        # As such, we consider the list matches if any of its contained tables matches each of the
-        # spec tables. This allows a dependency spec like so to match the torch cpu wheel in a lock
-        # that also includes torch-2.7.0-cp311-none-macosx_11_0_arm64.whl (cuda 12.8):
-        # {name = "torch", wheels = [{ name = "torch-2.7.0+xpu-cp39-cp39-win_amd64.whl" }]}
-        for spec_item in spec:
-            if not any(
-                spec_matches(spec_item, package_data_item) for package_data_item in package_data
-            ):
-                return False
-        return True
-
-    # I have no clue why MyPy can't track this as bool.
-    return cast(bool, spec == package_data)
-
-
 @attr.s
 class PackageParser(object):
     package_index = attr.ib()  # type: PackageIndex
     source = attr.ib()  # type: str
+    created_by = attr.ib()  # type: str
 
     parsed_packages_by_index = attr.ib(factory=dict)  # type: Dict[int, Package]
 
@@ -854,7 +892,7 @@ class PackageParser(object):
         return ArtifactURL.parse(url)
 
     def parse(self, indexed_package):
-        # type: (IndexedPackage) -> Union[Package, Error]
+        # type: (IndexedPackage) -> Union[Optional[Package], Error]
 
         index = indexed_package.index
         package = self.parsed_packages_by_index.get(index)
@@ -896,7 +934,22 @@ class PackageParser(object):
             "dependencies", default=[], diagnostic_key="name"
         )
         for dep_idx, dep_parse_context in enumerate(dep_parse_contexts):
-            dep_name = ProjectName(dep_parse_context.get_string("name"))
+            dep_name = ProjectName(
+                dep_parse_context.get_string(
+                    "name",
+                    footer_message=os.linesep.join(
+                        (
+                            "Pex requires dependency tables specify at least a `name`.",
+                            "The pylock.toml spec does not require this however.",
+                            "To subset locks created by {creator}, either they will need to add "
+                            "names for their dependencies or Pex will need to support more "
+                            "sophisticated dependency parsing.".format(creator=self.created_by),
+                            "For more information, see the comments starting here: "
+                            "https://github.com/pex-tool/pex/issues/2885#issuecomment-3263138568",
+                        )
+                    ),
+                )
+            )
 
             package_deps = self.package_index.packages(dep_name)
             if not package_deps:
@@ -906,31 +959,9 @@ class PackageParser(object):
                         project_name=project_name.raw, dep_name=dep_name.raw
                     )
                 )
-
-            deps = [
-                indexed_package
-                for indexed_package in package_deps
-                if spec_matches(dep_parse_context.table, indexed_package.package_data)
-            ]  # type: List[IndexedPackage]
-            if not deps:
-                return dep_parse_context.error(
-                    "No matching {dep_name} package could be found for {project_name} "
-                    "dependencies[{dep_idx}].".format(
-                        dep_name=dep_name.raw, project_name=project_name.raw, dep_idx=dep_idx
-                    )
-                )
-            elif len(deps) > 1:
-                return dep_parse_context.error(
-                    "More than one package matches {project_name} dependencies[{dep_idx}]:\n"
-                    "{matches}".format(
-                        project_name=project_name.raw,
-                        dep_idx=dep_idx,
-                        matches="\n".join(
-                            "+ packages[{index}]".format(index=dep.index) for dep in deps
-                        ),
-                    )
-                )
-            dependencies.append(Dependency(index=deps[0].index, project_name=dep_name))
+            dependencies.extend(
+                Dependency(index=dep.index, project_name=dep_name) for dep in package_deps
+            )
 
         vcs_parse_context = parse_context.get_table("vcs", default={})
         directory_parse_context = parse_context.get_table("directory", default={})
@@ -1097,7 +1128,12 @@ class PackageParser(object):
                         artifact = wheel_artifact
 
         if artifact is None:
-            return parse_context.error("Package must define an artifact.")
+            pex_warnings.warn(
+                "Skipping package at {path} in {lock} since it defines no artifacts.".format(
+                    path=parse_context.path, lock=parse_context.source
+                )
+            )
+            return None
 
         package = Package(
             index=index,
@@ -1127,12 +1163,16 @@ class PackageEvaluator(object):
         target,  # type: Target
         extras=frozenset(),  # type: FrozenSet[str]
         dependency_groups=frozenset(),  # type: FrozenSet[str]
+        requirements=(),  # type: Iterable[Requirement]
         constraints=(),  # type: Iterable[Requirement]
         build_configuration=BuildConfiguration(),  # type: BuildConfiguration
         dependency_configuration=DependencyConfiguration(),  # type: DependencyConfiguration
     ):
         # type: (...) -> PackageEvaluator
 
+        requirements_by_project_name = {
+            requirement.project_name: requirement for requirement in requirements
+        }
         constraints_by_project_name = {
             constraint.project_name: constraint.as_constraint() for constraint in constraints
         }
@@ -1165,6 +1205,7 @@ class PackageEvaluator(object):
             source=source,
             target=target,
             marker_environment=marker_environment,
+            requirements_by_project_name=requirements_by_project_name,
             constraints_by_project_name=constraints_by_project_name,
             build_configuration=build_configuration,
             dependency_configuration=dependency_configuration,
@@ -1173,6 +1214,7 @@ class PackageEvaluator(object):
     source = attr.ib()  # type: str
     target = attr.ib()  # type: Target
     marker_environment = attr.ib()  # type: Mapping[str, Union[str, FrozenSet[str]]]
+    requirements_by_project_name = attr.ib(factory=dict)  # type: Mapping[ProjectName, Requirement]
     constraints_by_project_name = attr.ib(factory=dict)  # type: Mapping[ProjectName, Constraint]
     build_configuration = attr.ib(default=BuildConfiguration())  # type: BuildConfiguration
     dependency_configuration = attr.ib(
@@ -1190,6 +1232,22 @@ class PackageEvaluator(object):
             return True
         return self.build_configuration.allow_build(package.project_name)
 
+    def _satisfies_requirements(
+        self,
+        project_name,  # type: ProjectName
+        version=None,  # type: Optional[Version]
+    ):
+        # type: (...) -> bool
+
+        if not version:
+            return True
+
+        requirement = self.requirements_by_project_name.get(project_name)
+        if not requirement:
+            return True
+
+        return requirement.contains(version)
+
     def _satisfies_constraints(
         self,
         project_name,  # type: ProjectName
@@ -1215,6 +1273,9 @@ class PackageEvaluator(object):
         if not self._satisfies_build_configuration(package):
             return False
 
+        if not self._satisfies_requirements(package.project_name, package.version):
+            return False
+
         if not self._satisfies_constraints(package.project_name, package.version):
             return False
 
@@ -1279,15 +1340,16 @@ class ResolvedPackages(object):
     def resolve_roots(self):
         # type: () -> Iterable[Package]
 
-        dependants_by_package_index = defaultdict(list)  # type: DefaultDict[int, List[Package]]
-        for package in self.packages:
-            if package.dependencies:
-                for dependency in package.dependencies:
-                    dependants_by_package_index[dependency.index].append(package)
-
-        return tuple(
-            package for package in self.packages if not dependants_by_package_index[package.index]
+        packages_by_index = {package.index: package for package in self.packages}
+        dag = tarjan.scc(
+            nodes=self.packages,
+            successors_fn=lambda package: tuple(
+                packages_by_index[dep.index]
+                for dep in (package.dependencies or ())
+                if dep.index in packages_by_index
+            ),
         )
+        return dag.roots()
 
 
 @attr.s(frozen=True)
@@ -1302,12 +1364,7 @@ class Pylock(object):
         except TomlDecodeError as e:
             return parse_context.error("Failed to parse TOML", e)
 
-        lock_version = Version(
-            parse_context.get_string(
-                "lock-version",
-                "Pex only supports lock version 1.0 and refuses to guess compatibility.",
-            )
-        )
+        lock_version = Version(parse_context.get_string("lock-version"))
         if lock_version != Version("1.0"):
             return parse_context.error(
                 "Found `lock-version` {version}, but Pex only supports version 1.0.".format(
@@ -1330,22 +1387,24 @@ class Pylock(object):
             "packages", default=[], diagnostic_key="name"
         )
         package_index = PackageIndex.create(packages_data)
-        package_parser = PackageParser(package_index=package_index, source=pylock_toml_path)
+        package_parser = PackageParser(
+            package_index=package_index, source=pylock_toml_path, created_by=created_by
+        )
 
         local_project_requirement_mapping = {}  # type: Dict[str, Requirement]
         packages = []  # type: List[Package]
         for indexed_package in package_index.iter_packages():
             package = try_(package_parser.parse(indexed_package))
+            if not package:
+                continue
             if isinstance(package.artifact, UnFingerprintedLocalProjectArtifact):
                 directory = package.artifact.directory
                 if not os.path.isabs(directory):
                     directory = os.path.normpath(
                         os.path.join(os.path.dirname(pylock_toml_path), directory)
                     )
-                local_project_requirement_mapping[directory] = Requirement.parse(
-                    "{project_name} @ file://{directory}".format(
-                        project_name=package.project_name, directory=directory
-                    )
+                local_project_requirement_mapping[directory] = Requirement.local(
+                    project_name=package.project_name, path=directory
                 )
             packages.append(package)
 
@@ -1454,6 +1513,7 @@ class Pylock(object):
             target=target,
             extras=extras,
             dependency_groups=groups,
+            requirements=requirements,
             constraints=constraints,
             build_configuration=build_configuration,
             dependency_configuration=dependency_configuration,
@@ -1461,7 +1521,6 @@ class Pylock(object):
         applicable_packages = [
             package for package in self.packages if package_evaluator.applies(package)
         ]
-
         packages_by_project_name = defaultdict(
             list
         )  # type: DefaultDict[ProjectName, List[Package]]
@@ -1469,22 +1528,36 @@ class Pylock(object):
             packages_by_project_name[package.project_name].append(package)
 
         if requirements:
+            marker_env = target.marker_environment.as_dict()
             required_project_names = deque(
-                OrderedSet(requirement.project_name for requirement in requirements)
+                OrderedSet(
+                    (requirement.project_name, str(requirement))
+                    for requirement in requirements
+                    if not requirement.marker or requirement.marker.evaluate(marker_env)
+                )
             )
             visited_projects = set()  # type: Set[ProjectName]
             packages = OrderedSet()  # type: OrderedSet[Package]
             while required_project_names:
-                project_name = required_project_names.popleft()
+                project_name, requirement = required_project_names.popleft()
                 if project_name in visited_projects:
                     continue
                 visited_projects.add(project_name)
                 selected_packages = packages_by_project_name[project_name]
+                if not selected_packages:
+                    return Error(
+                        "Failed to resolve a package satisfying {requirement} from "
+                        "{source}.".format(requirement=requirement, source=self.source)
+                    )
                 if transitive:
                     for selected_package in selected_packages:
                         if selected_package.dependencies:
                             for dep in selected_package.dependencies:
-                                required_project_names.append(dep.project_name)
+                                if dep.project_name not in packages_by_project_name:
+                                    continue
+                                required_project_names.append(
+                                    (dep.project_name, str(dep.project_name))
+                                )
                 packages.update(selected_packages)
 
             applicable_packages = list(packages)
