@@ -252,6 +252,7 @@ class ResolveUpdater(object):
         lock_configuration,  # type: LockConfiguration
         pip_configuration,  # type: PipConfiguration
         dependency_configuration,  # type: DependencyConfiguration
+        avoid_downloads,  # type: bool
     ):
         # type: (...) -> Union[ResolveUpdater, Error]
 
@@ -310,6 +311,7 @@ class ResolveUpdater(object):
             lock_configuration=lock_configuration,
             pip_configuration=pip_configuration,
             dependency_configuration=dependency_configuration,
+            avoid_downloads=avoid_downloads,
         )
 
     @classmethod
@@ -322,6 +324,7 @@ class ResolveUpdater(object):
         lock_configuration,  # type: LockConfiguration
         pip_configuration,  # type: PipConfiguration
         dependency_configuration,  # type: DependencyConfiguration
+        avoid_downloads,  # type: bool
     ):
         # type: (...) -> ResolveUpdater
 
@@ -362,6 +365,7 @@ class ResolveUpdater(object):
             lock_configuration=lock_configuration,
             pip_configuration=pip_configuration,
             dependency_configuration=dependency_configuration,
+            avoid_downloads=avoid_downloads,
         )
 
     requirement_configuration = attr.ib()  # type: RequirementConfiguration
@@ -377,6 +381,7 @@ class ResolveUpdater(object):
     update_requirements_by_project_name = attr.ib(
         factory=dict
     )  # type: Mapping[ProjectName, Requirement]
+    avoid_downloads = attr.ib(default=False)
 
     def iter_updated_requirements(self):
         # type: () -> Iterator[Requirement]
@@ -502,6 +507,7 @@ class ResolveUpdater(object):
                             targets=Targets.from_target(target),
                             pip_configuration=self.pip_configuration,
                             dependency_configuration=self.dependency_configuration,
+                            avoid_downloads=self.avoid_downloads,
                         )
                     )
                     assert 1 == len(updated_lock_file.locked_resolves)
@@ -691,6 +697,7 @@ class LockUpdater(object):
         use_pip_config,  # type: bool
         dependency_configuration,  # type: DependencyConfiguration
         pip_log,  # type: Optional[PipLog]
+        avoid_downloads,  # type: bool
     ):
         # type: (...) -> LockUpdater
 
@@ -711,12 +718,14 @@ class LockUpdater(object):
             lock_configuration=lock_file.configuration,
             pip_configuration=pip_configuration,
             dependency_configuration=dependency_configuration,
+            avoid_downloads=avoid_downloads,
         )
 
     lock_file = attr.ib()  # type: Lockfile
     lock_configuration = attr.ib()  # type: LockConfiguration
     pip_configuration = attr.ib()  # type: PipConfiguration
     dependency_configuration = attr.ib()  # type: DependencyConfiguration
+    avoid_downloads = attr.ib()  # type: bool
 
     def sync(
         self,
@@ -738,6 +747,7 @@ class LockUpdater(object):
                 lock_configuration=self.lock_configuration,
                 pip_configuration=self.pip_configuration,
                 dependency_configuration=self.dependency_configuration,
+                avoid_downloads=self.avoid_downloads,
             )
         )
         return self._perform_update(
@@ -773,6 +783,7 @@ class LockUpdater(object):
             lock_configuration=self.lock_configuration,
             pip_configuration=self.pip_configuration,
             dependency_configuration=self.dependency_configuration,
+            avoid_downloads=self.avoid_downloads,
         )
         return self._perform_update(
             update_requests=update_requests,
