@@ -39,6 +39,7 @@ from pex.venv.virtualenv import InstallationChoice, Virtualenv
 # Explicitly re-export subprocess to enable a transparent substitution in tests that supports
 # executing PEX files directly on Windows.
 from testing import subprocess as subprocess
+from testing.pytest_utils.tmp import Tempdir
 
 try:
     from unittest import mock
@@ -450,11 +451,12 @@ class IntegResults(object):
 def create_pex_command(
     args=None,  # type: Optional[Iterable[str]]
     python=None,  # type: Optional[str]
-    quiet=False,  # type: bool
+    quiet=None,  # type: Optional[bool]
+    pex_module="pex",  # type: str
 ):
     # type: (...) -> List[str]
-    cmd = [python or sys.executable, "-mpex"]
-    if not quiet:
+    cmd = [python or sys.executable, "-m", pex_module]
+    if pex_module == "pex" and not quiet:
         cmd.append("-v")
     if args:
         cmd.extend(args)
@@ -465,8 +467,9 @@ def run_pex_command(
     args,  # type: Iterable[str]
     env=None,  # type: Optional[Dict[str, str]]
     python=None,  # type: Optional[str]
-    quiet=False,  # type: bool
+    quiet=None,  # type: Optional[bool]
     cwd=None,  # type: Optional[str]
+    pex_module="pex",  # type: str
 ):
     # type: (...) -> IntegResults
     """Simulate running pex command for integration testing.
@@ -475,7 +478,7 @@ def run_pex_command(
     generated pex.  This is useful for testing end to end runs with specific command line arguments
     or env options.
     """
-    cmd = create_pex_command(args, python=python, quiet=quiet)
+    cmd = create_pex_command(args, python=python, quiet=quiet, pex_module=pex_module)
     process = Executor.open_process(
         cmd=cmd, env=env, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
     )
@@ -676,10 +679,11 @@ def ensure_python_venv(
     version,  # type: str
     latest_pip=True,  # type: bool
     system_site_packages=False,  # type: bool
+    tmpdir=None,  # type: Optional[Tempdir]
 ):
     # type: (...) -> Virtualenv
     pyenv_distribution = ensure_python_distribution(version)
-    venv = safe_mkdtemp()
+    venv = tmpdir.join("{version}.venv".format(version=version)) if tmpdir else safe_mkdtemp()
     if _ALL_PY_VERSIONS_TO_VERSION_INFO[version][0] == 3:
         args = [pyenv_distribution.binary, "-m", "venv", venv]
         if system_site_packages:
