@@ -38,7 +38,6 @@ from pex.typing import TYPE_CHECKING, cast
 from pex.venv.virtualenv import InstallationChoice, Virtualenv
 from testing import (
     IS_PYPY,
-    IS_X86_64,
     PY39,
     PY310,
     PY_VER,
@@ -1281,10 +1280,10 @@ def assert_p537_lock(
 
 
 skip_unless_p537_compatible = pytest.mark.skipif(
-    PY_VER < (3, 6) or PY_VER >= (3, 15) or IS_PYPY or not IS_X86_64,
+    PY_VER < (3, 6) or PY_VER >= (3, 16) or IS_PYPY,
     reason=(
-        "The p537 1.0.8 release only supports CPython >=3.6,<3.13 and only has published wheels "
-        "for Linux and Mac x86_64"
+        "The p537 1.0.10 release only supports CPython >=3.6,<3.16 and only has published wheels "
+        "for Linux and Mac."
     ),
 )
 
@@ -1314,7 +1313,7 @@ def test_sync_strict_to_strict(tmpdir):
 
     lock = os.path.join(str(tmpdir), "lock.json")
     run_pex3(
-        "lock", "sync", "--style", "strict", "p537==1.0.8", "--indent", "2", "--lock", lock
+        "lock", "sync", "--style", "strict", "p537==1.0.10", "--indent", "2", "--lock", lock
     ).assert_success()
     p537_current = assert_p537_lock(
         lock, LockStyle.STRICT, expected_python_tag=python_tag(), expected_abi_tag=abi_tag()
@@ -1333,7 +1332,7 @@ def test_sync_strict_to_strict(tmpdir):
         other_python,
         "--style",
         "strict",
-        "p537==1.0.8",
+        "p537==1.0.10",
         "--indent",
         "2",
         "--lock",
@@ -1355,7 +1354,7 @@ def test_sync_strict_to_sources(tmpdir):
 
     lock = os.path.join(str(tmpdir), "lock.json")
     run_pex3(
-        "lock", "sync", "--style", "strict", "p537==1.0.8", "--indent", "2", "--lock", lock
+        "lock", "sync", "--style", "strict", "p537==1.0.10", "--indent", "2", "--lock", lock
     ).assert_success()
     p537_strict = assert_p537_lock(
         lock, LockStyle.STRICT, expected_python_tag=python_tag(), expected_abi_tag=abi_tag()
@@ -1364,7 +1363,7 @@ def test_sync_strict_to_sources(tmpdir):
     p537_strict_tag, p537_strict_wheel = next(iter(p537_strict.artifacts_by_tag.items()))
 
     run_pex3(
-        "lock", "sync", "--style", "sources", "p537==1.0.8", "--indent", "2", "--lock", lock
+        "lock", "sync", "--style", "sources", "p537==1.0.10", "--indent", "2", "--lock", lock
     ).assert_success()
     p537_sources = assert_p537_lock(
         lock, LockStyle.SOURCES, expected_python_tag=python_tag(), expected_abi_tag=abi_tag()
@@ -1397,7 +1396,7 @@ def test_sync_universal_to_universal(
         py39.binary,
         "--interpreter-constraint",
         "CPython==3.9.*",
-        "p537==1.0.8",
+        "p537==1.0.10",
         "--indent",
         "2",
         "--lock",
@@ -1424,7 +1423,7 @@ def test_sync_universal_to_universal(
         py310.binary,
         "--interpreter-constraint",
         "CPython==3.10.*",
-        "p537==1.0.8",
+        "p537==1.0.10",
         "--indent",
         "2",
         "--lock",
