@@ -1018,7 +1018,7 @@ def test_multiplatform_entrypoint(tmpdir):
     interpreter = ensure_python_interpreter(PY39)
     res = run_pex_command(
         [
-            "p537==1.0.8",
+            "p537==1.0.10",
             "--no-build",
             "--python={}".format(interpreter),
             "--python-shebang=#!{}".format(interpreter),
@@ -1468,7 +1468,15 @@ def test_pex_cache_dir_and_pex_root():
         pex_file = os.path.join(td, "pex_file")
         run_pex_command(
             python=python,
-            args=["--cache-dir", cache_dir, "--pex-root", cache_dir, "p537==1.0.8", "-o", pex_file],
+            args=[
+                "--cache-dir",
+                cache_dir,
+                "--pex-root",
+                cache_dir,
+                "p537==1.0.10",
+                "-o",
+                pex_file,
+            ],
         ).assert_success()
 
         dists = list(iter_distributions(pex_root=cache_dir, project_name="p537"))
@@ -1480,7 +1488,7 @@ def test_pex_cache_dir_and_pex_root():
         # When the options have conflicting values they should be rejected.
         run_pex_command(
             python=python,
-            args=["--cache-dir", cache_dir, "--pex-root", pex_root, "p537==1.0.8", "-o", pex_file],
+            args=["--cache-dir", cache_dir, "--pex-root", pex_root, "p537==1.0.10", "-o", pex_file],
         ).assert_failure()
 
         assert not os.path.exists(cache_dir)
@@ -1495,7 +1503,7 @@ def test_disable_cache():
         pex_file = os.path.join(td, "pex_file")
         run_pex_command(
             python=python,
-            args=["--disable-cache", "p537==1.0.8", "-o", pex_file],
+            args=["--disable-cache", "p537==1.0.10", "-o", pex_file],
             env=make_env(PEX_ROOT=pex_root),
         ).assert_success()
 
