@@ -1,9 +1,8 @@
-# Copyright 2015 Pants project contributors (see CONTRIBUTORS.md).
+# Copyright 2015 Pex project contributors.
 # Licensed under the Apache License, Version 2.0 (see LICENSE).
 
 import os
 import shutil
-import subprocess
 import sys
 from textwrap import dedent
 
@@ -11,17 +10,21 @@ import pytest
 
 from pex.common import temporary_dir
 from pex.interpreter import PythonInterpreter
-from pex.interpreter_constraints import UnsatisfiableInterpreterConstraintsError
+from pex.interpreter_constraints import (
+    InterpreterConstraints,
+    UnsatisfiableInterpreterConstraintsError,
+)
 from pex.pex import PEX
 from pex.pex_bootstrapper import (
+    InterpreterTest,
     ensure_venv,
     find_compatible_interpreter,
     iter_compatible_interpreters,
 )
 from pex.pex_builder import PEXBuilder
-from pex.testing import PY27, PY37, PY310, ensure_python_interpreter
 from pex.typing import TYPE_CHECKING
 from pex.variables import ENV
+from testing import PY39, PY310, PY311, ensure_python_interpreter, subprocess
 
 if TYPE_CHECKING:
     from typing import Any, Iterable, List, Optional
@@ -35,16 +38,16 @@ def basenames(*paths):
 def find_interpreters(
     path,  # type: Iterable[str]
     valid_basenames=None,  # type: Optional[Iterable[str]]
-    constraints=None,  # type: Optional[Iterable[str]]
+    constraints=(),  # type: Iterable[str]
     preferred_interpreter=None,  # type: Optional[PythonInterpreter]
 ):
     # type: (...) -> List[str]
     return [
         interp.binary
         for interp in iter_compatible_interpreters(
-            path=os.pathsep.join(path),
+            path=tuple(path),
             valid_basenames=valid_basenames,
-            interpreter_constraints=constraints,
+            interpreter_constraints=InterpreterConstraints.parse(*constraints),
             preferred_interpreter=preferred_interpreter,
         )
     ]
@@ -52,32 +55,36 @@ def find_interpreters(
 
 def test_find_compatible_interpreters():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
+    py39 = ensure_python_interpreter(PY39)
     py310 = ensure_python_interpreter(PY310)
-    path = [py27, py37, py310]
+    py311 = ensure_python_interpreter(PY311)
+    path = [py39, py310, py311]
 
-    assert [py37, py310] == find_interpreters(path, constraints=[">3"])
-    assert [py27] == find_interpreters(path, constraints=["<3"])
+    assert [py310, py311] == find_interpreters(path, constraints=[">3.10"])
+    assert [py39] == find_interpreters(path, constraints=["<3.10"])
 
-    assert [py310] == find_interpreters(path, constraints=[">{}".format(PY37)])
-    assert [py37] == find_interpreters(path, constraints=[">{}, <{}".format(PY27, PY310)])
-    assert [py310] == find_interpreters(path, constraints=[">=3.10"])
+    assert [py311] == find_interpreters(path, constraints=[">{}".format(PY310)])
+    assert [py310] == find_interpreters(path, constraints=[">{}, <{}".format(PY39, PY311)])
+    assert [py311] == find_interpreters(path, constraints=[">=3.11"])
 
     with pytest.raises(UnsatisfiableInterpreterConstraintsError):
-        find_interpreters(path, constraints=["<2"])
+        find_interpreters(path, constraints=["<3"])
 
     with pytest.raises(UnsatisfiableInterpreterConstraintsError):
         find_interpreters(path, constraints=[">4"])
 
     with pytest.raises(UnsatisfiableInterpreterConstraintsError):
-        find_interpreters(path, constraints=[">{}, <{}".format(PY27, PY37)])
+        find_interpreters(path, constraints=[">{}, <{}".format(PY39, PY310)])
 
     # All interpreters on PATH including whatever interpreter is currently running.
-    all_known_interpreters = set(PythonInterpreter.all())
+    all_known_interpreters = set(PythonInterpreter.from_binary(python) for python in path)
     all_known_interpreters.add(PythonInterpreter.get())
 
-    interpreters = set(iter_compatible_interpreters(interpreter_constraints=["<3"]))
+    interpreters = set(
+        iter_compatible_interpreters(
+            path, interpreter_constraints=InterpreterConstraints.parse("<3.10")
+        )
+    )
     i_rendered = "\n      ".join(sorted(map(repr, interpreters)))
     aki_rendered = "\n      ".join(sorted(map(repr, all_known_interpreters)))
     assert interpreters.issubset(all_known_interpreters), dedent(
@@ -100,71 +107,71 @@ def test_find_compatible_interpreters_none():
 
 def test_find_compatible_interpreters_none_with_valid_basenames():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
-    path = [py27, py37]
+    py39 = ensure_python_interpreter(PY39)
+    py310 = ensure_python_interpreter(PY310)
+    path = [py39, py310]
 
     with pytest.raises(UnsatisfiableInterpreterConstraintsError) as exec_info:
         find_interpreters(path, valid_basenames=["python3.6"])
 
     exception_message = str(exec_info.value)
-    assert py27 not in exception_message
-    assert py37 not in exception_message
+    assert py39 not in exception_message
+    assert py310 not in exception_message
 
 
 def test_find_compatible_interpreters_none_with_constraints():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
-    path = [py27, py37]
+    py39 = ensure_python_interpreter(PY39)
+    py310 = ensure_python_interpreter(PY310)
+    path = [py39, py310]
 
     with pytest.raises(UnsatisfiableInterpreterConstraintsError) as exec_info:
-        find_interpreters(path, constraints=[">=3.8"])
+        find_interpreters(path, constraints=[">=3.11"])
 
     exception_message = str(exec_info.value)
-    assert py27 in exception_message
-    assert py37 in exception_message
-    assert ">=3.8" in exception_message
+    assert py39 in exception_message
+    assert py310 in exception_message
+    assert ">=3.11" in exception_message
 
 
 def test_find_compatible_interpreters_none_with_valid_basenames_and_constraints():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
-    path = [py27, py37]
+    py39 = ensure_python_interpreter(PY39)
+    py310 = ensure_python_interpreter(PY310)
+    path = [py310, py39]
 
     with pytest.raises(UnsatisfiableInterpreterConstraintsError) as exec_info:
-        find_interpreters(path, valid_basenames=basenames(py27), constraints=[">=3.6"])
+        find_interpreters(path, valid_basenames=basenames(py39), constraints=[">=3.10"])
 
     exception_message = str(exec_info.value)
-    assert py27 in exception_message
-    assert py37 not in exception_message
-    assert os.path.basename(py27) in exception_message, exception_message
-    assert ">=3.6" in exception_message
+    assert py39 in exception_message
+    assert os.path.basename(py39) in exception_message
+    assert py310 not in exception_message
+    assert ">=3.10" in exception_message
 
 
 def test_find_compatible_interpreters_with_valid_basenames():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
+    py39 = ensure_python_interpreter(PY39)
     py310 = ensure_python_interpreter(PY310)
-    path = [py27, py37, py310]
+    py311 = ensure_python_interpreter(PY311)
+    path = [py39, py310, py311]
 
-    assert [py37] == find_interpreters(path, valid_basenames=basenames(py37))
-    assert [py27, py310] == find_interpreters(
-        path, valid_basenames=basenames(*reversed([py27, py310]))
+    assert [py39] == find_interpreters(path, valid_basenames=basenames(py39))
+    assert [py310, py311] == find_interpreters(
+        path, valid_basenames=basenames(*reversed([py310, py311]))
     )
 
 
 def test_find_compatible_interpreters_with_valid_basenames_and_constraints():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
+    py39 = ensure_python_interpreter(PY39)
     py310 = ensure_python_interpreter(PY310)
-    path = [py27, py37, py310]
+    py311 = ensure_python_interpreter(PY311)
+    path = [py39, py310, py311]
 
-    assert [py37] == find_interpreters(
-        path, valid_basenames=basenames(py27, py37), constraints=[">=3"]
+    assert [py310] == find_interpreters(
+        path, valid_basenames=basenames(py39, py310), constraints=[">=3.10"]
     )
 
 
@@ -178,17 +185,17 @@ def test_find_compatible_interpreters_bias_current():
 
 def test_find_compatible_interpreters_siblings_of_current_issues_1109():
     # type: () -> None
-    py27 = ensure_python_interpreter(PY27)
+    py39 = ensure_python_interpreter(PY39)
     py310 = ensure_python_interpreter(PY310)
 
     with temporary_dir() as path_entry:
-        python27 = os.path.join(path_entry, "python2.7")
-        shutil.copy(py27, python27)
+        python39 = os.path.join(path_entry, "python3.9")
+        shutil.copy(py39, python39)
 
         python310 = os.path.join(path_entry, "python3.10")
         shutil.copy(py310, python310)
 
-        assert [os.path.realpath(p) for p in (python310, python27)] == find_interpreters(
+        assert [os.path.realpath(p) for p in (python310, python39)] == find_interpreters(
             path=[path_entry], preferred_interpreter=PythonInterpreter.from_binary(python310)
         )
 
@@ -206,7 +213,7 @@ def test_ensure_venv_activate_issues_1276(tmpdir):
     expected_python_bin_dir = (
         subprocess.check_output(
             args=[
-                venv_pex,
+                venv_pex.pex,
                 "-c",
                 "import os, sys; print(os.path.realpath(os.path.dirname(sys.executable)))",
             ],
@@ -214,7 +221,7 @@ def test_ensure_venv_activate_issues_1276(tmpdir):
         .decode("utf-8")
         .strip()
     )
-    bash_activate = os.path.join(os.path.dirname(venv_pex), "bin", "activate")
+    bash_activate = venv_pex.bin_file("activate")
 
     subprocess.check_call(
         args=[
@@ -264,15 +271,15 @@ def test_pp_exact():
 def test_pp_exact_on_ppp():
     # type: () -> None
 
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
+    py39 = ensure_python_interpreter(PY39)
     py310 = ensure_python_interpreter(PY310)
+    py311 = ensure_python_interpreter(PY311)
 
     with ENV.patch(
-        PEX_PYTHON=py310,
-        PEX_PYTHON_PATH=":".join(os.path.dirname(py) for py in (py27, py37, py310)),
+        PEX_PYTHON=py311,
+        PEX_PYTHON_PATH=os.pathsep.join(os.path.dirname(py) for py in (py39, py310, py311)),
     ):
-        assert PythonInterpreter.from_binary(py310) == find_compatible_interpreter()
+        assert PythonInterpreter.from_binary(py311) == find_compatible_interpreter()
 
 
 def test_pp_exact_satisfies_constraints():
@@ -280,9 +287,13 @@ def test_pp_exact_satisfies_constraints():
 
     py310 = ensure_python_interpreter(PY310)
 
+    pb = PEXBuilder()
+    pb.info.interpreter_constraints = InterpreterConstraints.parse(">=3.7")
+    pb.freeze()
+
     with ENV.patch(PEX_PYTHON=py310):
         assert PythonInterpreter.from_binary(py310) == find_compatible_interpreter(
-            interpreter_constraints=[">=3.7"]
+            interpreter_test=InterpreterTest(entry_point=pb.path(), pex_info=pb.info),
         )
 
 
@@ -291,26 +302,33 @@ def test_pp_exact_does_not_satisfy_constraints():
 
     py310 = ensure_python_interpreter(PY310)
 
+    pb = PEXBuilder()
+    pb.info.interpreter_constraints = InterpreterConstraints.parse("<=3.7")
+    pb.freeze()
+
     with ENV.patch(PEX_PYTHON=py310):
         with pytest.raises(
             UnsatisfiableInterpreterConstraintsError,
             match=r"Failed to find a compatible PEX_PYTHON={pp}.".format(pp=py310),
         ):
-            find_compatible_interpreter(interpreter_constraints=["<=3.7"])
+            find_compatible_interpreter(
+                interpreter_test=InterpreterTest(entry_point=pb.path(), pex_info=pb.info),
+            )
 
 
 def test_pp_exact_not_on_ppp():
     # type: () -> None
 
-    py27 = ensure_python_interpreter(PY27)
-    py37 = ensure_python_interpreter(PY37)
+    py39 = ensure_python_interpreter(PY39)
     py310 = ensure_python_interpreter(PY310)
+    py311 = ensure_python_interpreter(PY311)
 
     with ENV.patch(
-        PEX_PYTHON=py310, PEX_PYTHON_PATH=":".join(os.path.dirname(py) for py in (py27, py37))
+        PEX_PYTHON=py311,
+        PEX_PYTHON_PATH=os.pathsep.join(os.path.dirname(py) for py in (py39, py310)),
     ):
         with pytest.raises(
             UnsatisfiableInterpreterConstraintsError,
-            match=r"The specified PEX_PYTHON={pp} did not meet other constraints.".format(pp=py310),
+            match=r"The specified PEX_PYTHON={pp} did not meet other constraints.".format(pp=py311),
         ):
             find_compatible_interpreter()
